#include "avSG3GPP.h"
#include <utility.h>
#include <userint.h>

/*****************************************************************************/
/*   Copyright 1998 National Instruments Corporation.  All Rights Reserved.  */
/*****************************************************************************/


/*==================== R3562 3GPP Receiver Test Source ======================*/
/* LabWindows/CVI 5.0 Instrument Driver										 */
/* Original Release: February/15/2001										 */
/* By: Hitoshi Takahashi													 */
/*					 Supervisor: Yohei Hirakoso								 */
/*                   SPSpj Gyoda R&D,  ADVANTEST CORPORATION				 */
/*                   Home Page; http://www.advantest.co.jp					 */
/*                   e-mail;    info@advantest.co.jp						 */
/*---------------------------------------------------------------------------*/
/* Modification History: 													 */
/*         Version 1.0 :  February/15/2001									 */
/*																			 */
/*===========================================================================*/
 

#include <visa.h>
#include <ansi_c.h>
#include <formatio.h>
#include "avSG3GPP.h"

#define AVSG3GPP_REVISION    "Rev 1.0, 02/01, CVI 5.0" /*  Instrument driver revision */
#define BUFFER_SIZE         1024L 					  /* File I/O buffer size */
    

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/

/* General */
static ViString switchArr[] = {"OFF", "ON"};
static ViString dataPatternArr[] = {"PN9", "PN15", "ALL0", "ALL1", "PN9ERR"};
static ViString crcArr[] = {"NORMAL", "INVERSE", "ADDERR"};
static ViString polarityArr[] = {"NEG", "POS"}; 

/* Set Reference Level */
static ViString modeALCArr[] = {"ALCM AUTO", "ALCM, SH", "ALCM HOLD"};

/* Set Modulation */
static ViString directIQArr[] = {"IQDIR OFF", "IQDIR INPUT", "IQDIR OUTPUT"};
static ViString linkArr[] = {"LINK DN", "LINK UP"};

/* Set Up Link DPDCH */
static ViString upChnlCnfgArr[] = {"UPDPDCH:CCONF SP1", "UPDPDCH:CCONF SP2", 
								   "UPDPDCH:CCONF SP3", "UPDPDCH:CCONF SP4",
								   "UPDPDCH:CCONF SP5", "UPDPDCH:CCONF SP6", 
								   "UPDPDCH:CCONF SI2", "UPDPDCH:CCONF SI4",
								   "UPDPDCH:CCONF SI5", "UPDPDCH:CCONF SI6"};
								   
/* Set Down Link DPCH */
static ViString downChnlCnfgArr[] = {"DNDPCH:CCONF SP8", "DNDPCH:CCONF SP9",
									 "DNDPCH:CCONF SP10", "DNDPCH:CCONF SP11",
									 "DNDPCH:CCONF SP12", "DNDPCH:CCONF SP13",
									 "DNDPCH:CCONF SP14", "DNDPCH:CCONF SP15", 
									 "DNDPCH:CCONF SI11", "DNDPCH:CCONF SI13", 
									 "DNDPCH:CCONF SI14", "DNDPCH:CCONF SI15"};
									 
/* Start or Stop BER Measurement */
static ViString exeMeasArr[] = {"STOP", "BER"};

/* BER Counter Measurement */
static ViString dataBERArr[] = {"BMDAT PN15", "BMDAT PN9"};

/* Set Clock Timming */
static ViString syntheRefInArr[] = {"RSYN C0", "RSYN C1", "RSYN C2", "RSYN C3",
									"RSYN C4", "RSYN INTERNAL"};
static ViString modTimeBaseInArr[] = {"MODTB M0", "MODTB M1", "MODTB M2", "MODTB INTERNAL"};
static ViString clockOutArr[] = {"OFF", "CHIP", "RADIO", "SLOT", "TPCR", "TPCI"};

/* Save or Recall the Seting Conditions */
static ViString saveRecallArr[] = {"RECC", "SAVC"};

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* avSG3GPP_stringValPair is used in the avSG3GPP_errorMessage function        */
/*===========================================================================*/
typedef struct  avSG3GPP_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  avSG3GPP_tStringValPair;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean avSG3GPP_invalidViBooleanRange (ViBoolean val);
ViBoolean avSG3GPP_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean avSG3GPP_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean avSG3GPP_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max);
ViBoolean avSG3GPP_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max);
ViBoolean avSG3GPP_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean avSG3GPP_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max);
ViBoolean avSG3GPP_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus  avSG3GPP_initCleanUp (ViSession openRMSession, ViPSession openInstrSession,
								ViStatus currentStatus);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Initialize (ViRsrc resourceName, ViBoolean IDQuery,
                                       ViBoolean resetDevice, ViSession *instrSession)
{
    ViStatus avSG3GPP_status = VI_SUCCESS;
    ViSession rmSession = 0;
    ViUInt32 retCnt = 0;
    ViByte rdBuffer[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/
    if (avSG3GPP_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (avSG3GPP_invalidViBooleanRange (resetDevice))
        return VI_ERROR_PARAMETER3;

    /*- Open instrument session ---------------------------------------------*/
    if ((avSG3GPP_status = viOpenDefaultRM (&rmSession)) < 0)
        return avSG3GPP_status;

    if ((avSG3GPP_status = viOpen (rmSession, resourceName, VI_NULL, VI_NULL, instrSession)) < 0) {
        viClose (rmSession);
        return avSG3GPP_status;
    }

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    if ((avSG3GPP_status = viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 10000)) < 0)
            return avSG3GPP_initCleanUp (rmSession, instrSession, avSG3GPP_status);
  
    if ((avSG3GPP_status = viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000)) < 0)
            return avSG3GPP_initCleanUp (rmSession, instrSession, avSG3GPP_status);
  
    if ((avSG3GPP_status = viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return avSG3GPP_initCleanUp (rmSession, instrSession, avSG3GPP_status);
    
    if ((avSG3GPP_status = viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                            VI_FLUSH_ON_ACCESS)) < 0)
            return avSG3GPP_initCleanUp (rmSession, instrSession, avSG3GPP_status);

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery) 
    {
        if ((avSG3GPP_status = viWrite (*instrSession, "IDN?", 4, &retCnt)) < 0)
            return avSG3GPP_initCleanUp (rmSession, instrSession, avSG3GPP_status);
        if ((avSG3GPP_status = viRead (*instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
            return avSG3GPP_status;

        Scan (rdBuffer, "R3562");
        if (NumFmtdBytes () != 5) 
          	return avSG3GPP_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
    }
        
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice)
    {
        if ((avSG3GPP_status = avSG3GPP_Reset (*instrSession)) < 0)
            return avSG3GPP_initCleanUp (rmSession, instrSession, avSG3GPP_status);
    }       
          
    return avSG3GPP_status;
}

/*===========================================================================*/
/* Function: Set Output Frequency                                            */
/* Purpose:  This function sets the output frequency.						 */
/*			 The output frequency is determined by frequency directry, or	 */
/*			 channel setting.												 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Set_OutputFrequency (ViSession instrSession,
                                               ViBoolean setMode, ViReal64 outFreq,
                                               ViInt16 outChnl, ViReal64 strFreq,
                                               ViReal64 chnlSpc, ViInt16 strChnlNum)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    ViUInt32 total = 0;
    
    if (avSG3GPP_invalidViBooleanRange (setMode))             
    	return VI_ERROR_PARAMETER2;					
    if (avSG3GPP_invalidViReal64Range (outFreq, 800.0e6, 2.3e9))             
    	return VI_ERROR_PARAMETER3;
    if (avSG3GPP_invalidViReal64Range (strFreq, 800.0e6, 2.3e9))             
    	return VI_ERROR_PARAMETER5;
        
    Fmt (wrtBuffer, "%s<CSF %fHZ; CSP %fHZ; CSN %d", outFreq, chnlSpc, strChnlNum);
    total += NumFmtdBytes();
    
    if (setMode==0)  /* Set Channel Number */
    	Fmt (wrtBuffer, "%s[a]<; CH %d", outChnl);
    else			 /* Set Frequency */
    	Fmt (wrtBuffer, "%s[a]<; FR %fHZ", outFreq);
    total += NumFmtdBytes();
    
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,total,&retCount)) < 0)
    	return avSG3GPP_status;
        
    return avSG3GPP_status;
}

/*===========================================================================*/
/* Function: Set Reference Level                                             */
/* Purpose:  This function sets the reference level parameters.				 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Set_ReferenceLevel (ViSession instrSession,
                                              ViBoolean output, ViReal64 refLevel,
                                              ViReal64 upLmtLevel, ViBoolean lvlOffset,
                                              ViReal64 lvlOffsetVal, ViInt16 modeALC)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
	ViByte wrtBuffer[BUFFER_SIZE];
	ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViBooleanRange (output))             
    	return VI_ERROR_PARAMETER2;	
    if (avSG3GPP_invalidViReal64Range (refLevel, -125.0, 0.0))             
    	return VI_ERROR_PARAMETER3;
    if (avSG3GPP_invalidViReal64Range (upLmtLevel, -125.0, 0.0))             
    	return VI_ERROR_PARAMETER4;
    if (avSG3GPP_invalidViBooleanRange (lvlOffset))             
    	return VI_ERROR_PARAMETER5;	
    if (avSG3GPP_invalidViInt16Range (modeALC, 0, 2))             
    	return VI_ERROR_PARAMETER7;
    	
    Fmt (wrtBuffer, "%s<OUT %s; OLM %fDM; AP %fDM; OOF %s; OOS %fDB; %s",
    	 switchArr[output], upLmtLevel, refLevel, switchArr[lvlOffset], 
    	 lvlOffsetVal, modeALCArr[modeALC]);
    
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;	 
    	
	return avSG3GPP_status;
}

/*===========================================================================*/
/* Function: Set Modulation                                             	 */
/* Purpose:  This function sets the modultion parameters.					 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Set_Modulation (ViSession instrSession,
                                           ViBoolean mdl, ViInt16 drct, 
                                           ViBoolean link)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
	ViByte wrtBuffer[BUFFER_SIZE];
	ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViBooleanRange (mdl))             
    	return VI_ERROR_PARAMETER2;	
    if (avSG3GPP_invalidViInt16Range (drct, 0, 2))             
    	return VI_ERROR_PARAMETER3;
    if (avSG3GPP_invalidViBooleanRange (link))             
    	return VI_ERROR_PARAMETER4;
    	
    Fmt (wrtBuffer, "%s<MOD %s; %s; %s", switchArr[mdl], directIQArr[drct], linkArr[link]);
    
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;	 
    	
	return avSG3GPP_status;    
}

/*===========================================================================*/
/* Function: Set Up Link DPDCH                                    			 */
/* Purpose:  This function sets the DPDCH paramenters in the up link mode.	 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Set_UpLinkDPDCH (ViSession instrSession,
                                            ViInt16 chnlCnfg, ViInt16 dataDPDCH,
                                            ViInt16 dataDTCH, ViBoolean fecDTCH,
                                            ViInt16 crcDTCH, ViInt16 dataDCCH,
                                            ViBoolean fecDCCH, ViInt16 crcDCCH)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
	ViByte wrtBuffer[BUFFER_SIZE];
	ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViInt16Range (chnlCnfg, 0, 9))             
    	return VI_ERROR_PARAMETER2;
    if (avSG3GPP_invalidViInt16Range (dataDPDCH, 0, 4))             
    	return VI_ERROR_PARAMETER3;	
    if (avSG3GPP_invalidViInt16Range (dataDTCH, 0, 4))             
    	return VI_ERROR_PARAMETER4;	
    if (avSG3GPP_invalidViBooleanRange (fecDTCH))             
    	return VI_ERROR_PARAMETER5;	
    if (avSG3GPP_invalidViInt16Range (crcDTCH, 0, 2))             
    	return VI_ERROR_PARAMETER6;	
    if (avSG3GPP_invalidViInt16Range (dataDCCH, 0, 4))             
    	return VI_ERROR_PARAMETER7;	
    if (avSG3GPP_invalidViBooleanRange (fecDCCH))             
    	return VI_ERROR_PARAMETER8;	
    if (avSG3GPP_invalidViInt16Range (crcDCCH, 0, 2))             
    	return VI_ERROR_INSTR_PARAMETER9;
    	
    Fmt (wrtBuffer, "%s<%s; UPDPDCH:DATA %s; UPDTCH:DATA %s; UPDTCH:FEC %s; UPDTCH:CRC %s; UPDCCH:DATA %s; UPDCCH:FEC %s; UPDCCH:CRC %s",
    	 upChnlCnfgArr[chnlCnfg], dataPatternArr[dataDPDCH],
    	 dataPatternArr[dataDTCH], switchArr[fecDTCH], crcArr[crcDTCH],
    	 dataPatternArr[dataDCCH], switchArr[fecDCCH], crcArr[crcDCCH]);
    
	if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;	 
    	
	return avSG3GPP_status;    
}

/*===========================================================================*/
/* Function: Set Up Link DPCCH                                    			 */
/* Purpose:  This function sets the DPCCH paramenters in the up link mode.	 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Set_UpLinkDPCCH (ViSession instrSession,
                                            ViInt16 sltFmt, ViInt16 codeTFCI,
                                            ViInt32 bitPttnFBI, ViInt16 sltLenTPC,
                                            ViInt16 insrtSltLenTPC)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViInt16Range (sltFmt, 0, 5))             
    	return VI_ERROR_PARAMETER2;
    if (avSG3GPP_invalidViInt16Range (codeTFCI, 0x0, 0x3FF))             
    	return VI_ERROR_PARAMETER3;
    if (avSG3GPP_invalidViInt32Range (bitPttnFBI, 0x0, 0x3FFFFFFF))             
    	return VI_ERROR_PARAMETER4;
    if (avSG3GPP_invalidViInt16Range (sltLenTPC, 1, 75))             
    	return VI_ERROR_PARAMETER5;
    if (avSG3GPP_invalidViInt16Range (insrtSltLenTPC, -75, 75))             
    	return VI_ERROR_PARAMETER6;	
    if (insrtSltLenTPC==0)
    	return VI_ERROR_PARAMETER6;
    	
    Fmt (wrtBuffer, "%s<UPDPCCH:PFORM %d; UPDPCCH:TFCI %x; UPDPCCH:FBI %x; UPDPCCH:TPCR %d; TPCI %d",
    	 sltFmt, codeTFCI, bitPttnFBI, sltLenTPC, insrtSltLenTPC);
    
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;	 
    	
	return avSG3GPP_status;
}

/*===========================================================================*/
/* Function: Set Up Link Spreading                                 			 */
/* Purpose:  This function sets the scrambling code in the up link mode.	 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_set_UpLinkSpreading (ViSession instrSession,
                                                ViInt32 scrmblCode)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViInt32Range (scrmblCode, 0, 16777215))             
    	return VI_ERROR_PARAMETER2;
    	
    Fmt (wrtBuffer, "%s<UPSCODE %d", scrmblCode);
    
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;	 
    	
	return avSG3GPP_status;
}

/*===========================================================================*/
/* Function: Set Up Link Gain Parameters                           			 */
/* Purpose:  This function sets the gain parameters in the up link mode.	 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Set_UpLinkGainParameters (ViSession instrSession,
                                                     ViInt16 cntrlSlct,
                                                     ViInt16 gainParaDPDCH, ViInt16 gainParaDPCCH,
                                                     ViReal64 pwrRateDPDCH, ViReal64 pwrRateDPCCH)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViInt16Range (cntrlSlct, 0, 3))             
    	return VI_ERROR_PARAMETER2;
    if (avSG3GPP_invalidViInt16Range (gainParaDPDCH, 0, 15))             
    	return VI_ERROR_PARAMETER3;
    if (avSG3GPP_invalidViInt16Range (gainParaDPCCH, 0, 15))             
    	return VI_ERROR_PARAMETER4;
    if (avSG3GPP_invalidViReal64Range (pwrRateDPDCH, -99.9, 0.0))             
    	return VI_ERROR_PARAMETER5;	
    if (pwrRateDPDCH < -23.5 && pwrRateDPDCH > -99.9)
    	return VI_ERROR_PARAMETER5;
    if (avSG3GPP_invalidViReal64Range (pwrRateDPCCH, -99.9, 0.0))             
    	return VI_ERROR_PARAMETER6;	
    if (pwrRateDPCCH < -23.5 && pwrRateDPCCH > -99.9)
    	return VI_ERROR_PARAMETER6;
    
    switch (cntrlSlct)
    {
    case 0:   /* Set DPDCH's Parameter Code */
    	Fmt (wrtBuffer, "%s<UPDPDCH:GAINC %d", gainParaDPDCH);
    	break;
    	
    case 1:   /* Set DPCCH's Parameter Code */
    	Fmt (wrtBuffer, "%s<UPDPCCH:GAINC %d", gainParaDPCCH);
    	break;
    	
    case 2:   /* Set DPDCH's Power Rate */
    	Fmt (wrtBuffer, "%s<UPDPDCH:GAINP %f", pwrRateDPDCH);
    	break;
    	
    case 3:   /* Set DPCCH's Power Rate */
    	Fmt (wrtBuffer, "%s<UPDPCCH:GAINP %f", pwrRateDPCCH);
    	break;
    }
    
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;	 
    	
	return avSG3GPP_status;
}

/*===========================================================================*/
/* Function: Set Down Link DPCH			                           			 */
/* Purpose:  This function sets the DPCH parameters in the down link mode.	 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Set_DownLinkDPCH (ViSession instrSession,
                                             ViInt16 chnlConfig)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViInt16Range (chnlConfig, 0, 11))             
    	return VI_ERROR_PARAMETER2;
    	
    Fmt (wrtBuffer, "%s<%s", downChnlCnfgArr[chnlConfig]);
    
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;	 
    	
	return avSG3GPP_status;
}

/*===========================================================================*/
/* Function: Set Down Link DPDCH		                           			 */
/* Purpose:  This function sets the DPDCH parameters in the down link mode.	 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Set_DownLinkDPDCH (ViSession instrSession, ViInt16 dataDPDCH,
                                              ViInt16 dataDTCH, ViBoolean fecDTCH,
                                              ViInt16 crcDTCH, ViInt16 dataDCCH,
                                              ViBoolean fecDCCH, ViInt16 crcDCCH)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViInt16Range (dataDPDCH, 0, 4))             
    	return VI_ERROR_PARAMETER2;
    if (avSG3GPP_invalidViInt16Range (dataDTCH, 0, 4))             
    	return VI_ERROR_PARAMETER3;
    if (avSG3GPP_invalidViBooleanRange (fecDTCH))             
    	return VI_ERROR_PARAMETER4;
    if (avSG3GPP_invalidViInt16Range (crcDTCH, 0, 2))             
    	return VI_ERROR_PARAMETER5;
    if (avSG3GPP_invalidViInt16Range (dataDCCH, 0, 4))             
    	return VI_ERROR_PARAMETER6;
    if (avSG3GPP_invalidViBooleanRange (fecDTCH))             
    	return VI_ERROR_PARAMETER7;
    if (avSG3GPP_invalidViInt16Range (crcDCCH, 0, 2))             
    	return VI_ERROR_PARAMETER8;
    	
    Fmt (wrtBuffer, "%s<DNDPDCH:DATA %s; DNDTCH:DATA %s; DNDTCH:FEC %s; DNDTCH:CRC %s; DNDCCH:DATA %s; DNDCCH:FEC %s; DNDCCH:CRC %s",
    	 dataPatternArr[dataDPDCH], dataPatternArr[dataDTCH], switchArr[fecDTCH], crcArr[crcDTCH],
    	 dataPatternArr[dataDCCH], switchArr[fecDCCH], crcArr[crcDCCH]);
    	 
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;	 
    	
	return avSG3GPP_status;	
}

/*===========================================================================*/
/* Function: Set Down Link DPCCH		                           			 */
/* Purpose:  This function sets the DPCCH parameters in the down link mode.	 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Set_DownLinkDPCCH (ViSession instrSession, ViInt16 codeTFCI,
                                              ViInt16 sltLenTPC, ViInt16 insrtSltLenTPC)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViInt16Range (codeTFCI, 0x0, 0x3FF))             
    	return VI_ERROR_PARAMETER2;
    if (avSG3GPP_invalidViInt16Range (sltLenTPC, 1, 75))             
    	return VI_ERROR_PARAMETER3;
    if (avSG3GPP_invalidViInt16Range (insrtSltLenTPC, -75, 75))             
    	return VI_ERROR_PARAMETER4;	
    if (insrtSltLenTPC==0)
    	return VI_ERROR_PARAMETER4;
    	
    Fmt (wrtBuffer, "%s<DNDPCCH:TFCI %x; DNDPCCH:TPCR %d; TPCI %d",
    	 codeTFCI, sltLenTPC, insrtSltLenTPC);
    
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;	 
    	
	return avSG3GPP_status;	 
}

/*===========================================================================*/
/* Function: Set Down Link Spreading	                           			 */
/* Purpose:  This function sets the Scrambling Code in the down link mode.	 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Set_DownLinkSpreading (ViSession instrSession,
                                                  ViInt16 chnlCode, ViInt16 scrmblCode)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViInt16Range (chnlCode, 2, 127))             
    	return VI_ERROR_PARAMETER2;
    if (avSG3GPP_invalidViInt16Range (scrmblCode, 0, 8191))             
    	return VI_ERROR_PARAMETER3;
        
    Fmt (wrtBuffer, "%s<DNDPCH:CCODE %d; DNSCODE %d", chnlCode, scrmblCode);
    
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;	 
    	
	return avSG3GPP_status;	 
}

/*===========================================================================*/
/* Function: Set Down Link Channel Power                           			 */
/* Purpose:  This function sets CPICH, P-CCPCH and DPCH channel's power. 	 */
/*			 Each channel's power is calculated and set, holding each 		 */
/*			 channel's power difference as setting value,  and holding total */
/*			 power constant.												 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Set_DownLinkChannelPower (ViSession instrSession, ViReal64 pwrRateCPICH,
                                                     ViReal64 pwrRatePCCPCH, ViReal64 pwrRateDPCH)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViReal64Range (pwrRateCPICH, -99.9, 0.0))             
    	return VI_ERROR_PARAMETER2;
    if (pwrRateCPICH < -20.0 && pwrRateCPICH > -99.9)
    	return VI_ERROR_PARAMETER2;
    if (avSG3GPP_invalidViReal64Range (pwrRatePCCPCH, -99.9, 0.0))             
    	return VI_ERROR_PARAMETER3;
    if (pwrRatePCCPCH < -20.0 && pwrRatePCCPCH > -99.9)
    	return VI_ERROR_PARAMETER3;
    if (avSG3GPP_invalidViReal64Range (pwrRateDPCH, -99.9, 0.0))             
    	return VI_ERROR_PARAMETER4;
    if (pwrRateDPCH < -20.0 && pwrRateDPCH > -99.9)
    	return VI_ERROR_PARAMETER4;
    	
    Fmt (wrtBuffer, "%s<DNCPICH:GAINP %f", pwrRateCPICH);
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;
    	
    Fmt (wrtBuffer, "%s<DNPCCPCH:GAINP %f", pwrRatePCCPCH);
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;
    	
    Fmt (wrtBuffer, "%s<DNDPCH:GAINP %f", pwrRateDPCH);
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;	
    	
    return avSG3GPP_status;	
}

/*===========================================================================*/
/* Function: Adjust External IQ			                           			 */
/* Purpose:  This function adjusts the exterrnal IQ signal.					 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Adjust_ExternalIQ (ViSession instrSession,
                                              ViInt16 inputGainI, ViInt16 inputGainQ,
                                              ViInt16 inputPhase,
                                              ViInt16 outputGainI, ViInt16 outputGainQ)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViInt16Range (inputGainI, 40, 4000))             
    	return VI_ERROR_PARAMETER2;
    if (avSG3GPP_invalidViInt16Range (inputGainQ, 40, 4000))             
    	return VI_ERROR_PARAMETER3;
    if (avSG3GPP_invalidViInt16Range (inputPhase, 40, 4000))             
    	return VI_ERROR_PARAMETER4;
    if (avSG3GPP_invalidViInt16Range (outputGainI, -700, 700))             
    	return VI_ERROR_PARAMETER5;
    if (avSG3GPP_invalidViInt16Range (outputGainQ, -700, 700))             
    	return VI_ERROR_PARAMETER6;	
        
    Fmt (wrtBuffer, "%s<LBAI %d; LBAQ %d; PHA %d; LOAI %d; LOAQ %d",
    	 inputGainI, inputGainQ, inputPhase, outputGainI, outputGainQ);
    
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;	
    	
    return avSG3GPP_status;		 
}

/*===========================================================================*/
/* Function: Start or Stop BER Measurement                         			 */
/* Purpose:  This function starts or stops BER measurement.					 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_StartStop_BERCounter (ViSession instrSession,
                                                 ViBoolean measurement)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViBooleanRange (measurement))             
    	return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuffer, "%s<%s", exeMeasArr[measurement]);
    
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;	
    	
    return avSG3GPP_status;	
}

/*===========================================================================*/
/* Function: BER Counter Measurement	                         			 */
/* Purpose:  This function sets the bit error rate measurement's 			 */
/*			 parameters, and executes messurement.							 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Meas_BERCounter (ViSession instrSession,
                                            ViBoolean exeMeas, ViBoolean measData,
                                            ViInt32 bitLength,
                                            ViBoolean clkPolarity, ViBoolean dataPolarity)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViBooleanRange (exeMeas))             
    	return VI_ERROR_PARAMETER2;
    if (avSG3GPP_invalidViBooleanRange (measData))             
    	return VI_ERROR_PARAMETER3;
    if (avSG3GPP_invalidViInt32Range (bitLength, 1000, 10000000))             
    	return VI_ERROR_PARAMETER4;
    if (avSG3GPP_invalidViBooleanRange (clkPolarity))             
    	return VI_ERROR_PARAMETER5;
    if (avSG3GPP_invalidViBooleanRange (dataPolarity))             
    	return VI_ERROR_PARAMETER6;
    
    Fmt (wrtBuffer, "%s<%s; BLEN %d; BCLK %s; BDAT %s; %s",
    	 dataBERArr[measData], bitLength, polarityArr[clkPolarity],
    	 polarityArr[dataPolarity], exeMeasArr[exeMeas]);
        
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;	
    	
    return avSG3GPP_status;	    
}

/*===========================================================================*/
/* Function: Calibration				                         			 */
/* Purpose:  This function sets the calibration correction, and executes	 */
/*			 calibration.							 						 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Calibration (ViSession instrSession,
                                        ViBoolean exeCalib, ViBoolean calibCrct)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    ViUInt32 total = 0;
    
    if (avSG3GPP_invalidViBooleanRange (exeCalib))             
    	return VI_ERROR_PARAMETER2;
    if (avSG3GPP_invalidViBooleanRange (calibCrct))             
    	return VI_ERROR_PARAMETER3;
    	
    Fmt (wrtBuffer, "%s<CMC %s", switchArr[calibCrct]);
    total += NumFmtdBytes();
    
    if (exeCalib==VI_ON)
    {
    	Fmt (wrtBuffer, "%s[a]<; CMOD");
    	total += NumFmtdBytes();
    }
    
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,total,&retCount)) < 0)
    	return avSG3GPP_status;	
    	
    return avSG3GPP_status;	    
}

/*===========================================================================*/
/* Function: Set Clock Timming			                         			 */
/* Purpose:  This function sets the parameters of the clock and timing 		 */
/*			 signal that are input from or output to the connectors of the   */
/*			 instrument.													 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Set_ClockTiming (ViSession instrSession,
                                            ViInt16 refIn, ViInt16 refSrcAdjust,
                                            ViInt16 modTmBase, ViBoolean extTrgCmnd,
                                            ViBoolean extTrgPlty, ViInt16 extTrgOfst,
                                            ViInt16 clockOut1, ViInt16 clockOut2)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    ViUInt32 total = 0;
    
    if (avSG3GPP_invalidViInt16Range (refIn, 0, 5))             
    	return VI_ERROR_PARAMETER2;
    if (avSG3GPP_invalidViInt16Range (refSrcAdjust, -2000, 2000))             
    	return VI_ERROR_PARAMETER3; 
    if (avSG3GPP_invalidViInt16Range (modTmBase, 0, 3))             
    	return VI_ERROR_PARAMETER4; 
    if (avSG3GPP_invalidViBooleanRange (extTrgCmnd))             
    	return VI_ERROR_PARAMETER5; 
    if (avSG3GPP_invalidViBooleanRange (extTrgPlty))             
    	return VI_ERROR_PARAMETER6; 
    if (avSG3GPP_invalidViInt16Range (extTrgOfst, 20, 200))             
    	return VI_ERROR_PARAMETER7; 
    if (avSG3GPP_invalidViInt16Range (clockOut1, 0, 5))             
    	return VI_ERROR_PARAMETER8; 
    if (avSG3GPP_invalidViInt16Range (clockOut2, 0, 5))             
    	return VI_ERROR_INSTR_PARAMETER9;
    	
    Fmt (wrtBuffer, "%s<%s; SRAD %d; %s; ETRGPOL %s; ETRGOFFSET %d; REAROUT1 %s; REAROUT2 %s",
    	 syntheRefInArr[refIn], refSrcAdjust, modTimeBaseInArr[modTmBase],
    	 polarityArr[extTrgPlty], extTrgOfst, clockOutArr[clockOut1], clockOutArr[clockOut2]);
    total += NumFmtdBytes();
    
    if (extTrgCmnd==VI_ON)
    {
    	Fmt (wrtBuffer, "%s[a]<; ETRG");
    	total += NumFmtdBytes();
    }
    
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,total,&retCount)) < 0)
    	return avSG3GPP_status;	
    	
    return avSG3GPP_status;	   
}

/*===========================================================================*/
/* Function: Save or Recall the Setting Conditions                			 */
/* Purpose:  This functions saves or recalls the setting conditions using	 */
/*			 instrument's backup memory.									 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_SaveRecall (ViSession instrSession,
                                       ViBoolean operate, ViInt16 num)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViBooleanRange (operate))             
    	return VI_ERROR_PARAMETER2;
    if (avSG3GPP_invalidViInt16Range (num, 1, 3))             
    	return VI_ERROR_PARAMETER3;
    	
    Fmt (wrtBuffer, "%s<%s %d", saveRecallArr[operate], num);
    
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;	
    	
    return avSG3GPP_status;	   
}

/*===========================================================================*/
/* Function: Status Byte Register					               			 */
/* Purpose:  The status byte register sumarizes the information from the	 */
/*			 status register. In addition, a summary of the status byte		 */
/*			 register is sent to the controller as a service request.		 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_StatusByteRegister (ViSession instrSession,
                                               ViBoolean regMode,
                                               ViInt16 regVal, ViInt16 *regStt)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte Buffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViBooleanRange (regMode))             
    	return VI_ERROR_PARAMETER2;
    if (avSG3GPP_invalidViInt16Range (regVal, 0, 256))             
    	return VI_ERROR_PARAMETER3;	
    
    if (regMode==VI_OFF)   /* Read Mode */
    {
    	if ((avSG3GPP_status = viWrite(instrSession, "*STB?", 5, &retCount))<0)
        	return avSG3GPP_status;
        if ((avSG3GPP_status = viRead (instrSession, Buffer, 5, &retCount)) < 0)
        	return avSG3GPP_status;
    	if (Scan(Buffer,"%s>%d[b2]", regStt)!=1) 
    	{ 
        	avSG3GPP_status = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        	return avSG3GPP_status;
        }	
    }
    
    else				   /* Write Mode */
    {
    	Fmt (Buffer, "%s<*SRE %d", regVal);
    	if ((avSG3GPP_status = viWrite(instrSession, Buffer, NumFmtdBytes(), &retCount)) < 0)
    		return avSG3GPP_status;	
    }
    
    return avSG3GPP_status;
}

/*===========================================================================*/
/* Function: Service Request ON OFF                                          */
/* Purpose:  This function controls service request signal.                  */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_ServiceRequest (ViSession instrSession, ViBoolean srq)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViByte wrtBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if (avSG3GPP_invalidViBooleanRange (srq))             
    	return VI_ERROR_PARAMETER2;
    	
    Fmt (wrtBuffer, "%s<SRQ %d", srq);	
    
    if ((avSG3GPP_status = viWrite(instrSession,wrtBuffer,NumFmtdBytes(),&retCount)) < 0)
    	return avSG3GPP_status;	
    	
    return avSG3GPP_status;	 
}

/*===========================================================================*/
/* Function: Clear Status Byte		                                         */
/* Purpose:  This function clears status byte.				                 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Clear_StatusByte (ViSession instrSession)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
    ViUInt32 retCount = 0;
    
	if ((avSG3GPP_status = viWrite(instrSession, "CSB", 3, &retCount)) < 0)
    	return avSG3GPP_status;	
    	
    return avSG3GPP_status;	    
}

/*===========================================================================*/
/* Function: Wait for Execution                                              */
/* Purpose:  When BER measurement or calibration is started, this function   */
/*			 waits for some accident to occur. Some accidents are that BER   */
/*			 measurement or calibration is stopped, or executing time is	 */
/*			 reached the setting maximum wait time.							 */
/*===========================================================================*/
ViInt32 strng_sec(char *tm)                   
{                                             
	int a,b,c;                                
	ViInt32 vSec;                             
	                                          
	Scan (tm, "%s>%d:%d:%d", &a, &b, &c);         
	vSec = a*3600 + b*60 + c;                 
	return vSec;                              
}                                             

ViStatus _VI_FUNC avSG3GPP_Wait_Execution (ViSession instrSession,
                                           ViInt32 maxTime, ViBoolean *errFlag)
{
	ViUInt32 retCnt = 0;                  
    ViChar rdBuffer[BUFFER_SIZE];           
    ViStatus avSG3GPP_status = VI_SUCCESS;
    ViInt16 flag = 0;
    ViInt16 status;
    ViInt32 strSec, stpSec, realSec;
    
    if (avSG3GPP_invalidViInt32Range (maxTime, 0, 3600))           
        return VI_ERROR_PARAMETER2;                        
    
	strSec = strng_sec (TimeStr());
    stpSec = strSec + maxTime; 
    
    while (flag==0)
    {
    	if ((avSG3GPP_status = viWrite (instrSession, "*STB?", 5, &retCnt)) < 0)       
    	    return avSG3GPP_status;                                                    
    	if ((avSG3GPP_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCnt)) < 0)
    	    return avSG3GPP_status;                                                    
    	if (Scan (rdBuffer, "%s>%d[b2]", &status) < 1)                             
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;                               
    
    	realSec = strng_sec (TimeStr());
    	if (realSec < strSec)
    		realSec += 86400;
    	
    	if (status!=0 || realSec>stpSec)
    		flag = 1;
    	
		Delay (2.0e-3);
    	
    }
    
    if (status==1 || status==8)
    	*errFlag = VI_OFF;
    else
    	*errFlag = VI_ON;
    
	return avSG3GPP_status;    
}

/*===========================================================================*/
/* Function: Read Ber Counter Data                                           */
/* Purpose:  This function returns the BER (bit error rate) data.			 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Read_BERCounter (ViSession instrSession,
                                            ViReal64 *errRate)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
	ViChar rdBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avSG3GPP_status = viWrite (instrSession, "BER?", 4, &retCount)) < 0)                
	    return avSG3GPP_status;                                                            
	if ((avSG3GPP_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avSG3GPP_status;                                                            
	if (Scan (rdBuffer, "%s>%f", errRate) < 1)                 
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	return avSG3GPP_status;    
}

/*===========================================================================*/
/* Function: Read Meas Status Register                                       */
/* Purpose:  This function returns the measurement status register's value.  */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Read_MeasStatusRegister (ViSession instrSession,
                                                    ViInt16 *reg)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
	ViChar rdBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avSG3GPP_status = viWrite (instrSession, "MST?", 4, &retCount)) < 0)                
	    return avSG3GPP_status;                                                            
	if ((avSG3GPP_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avSG3GPP_status;                                                            
	if (Scan (rdBuffer, "%s>%d[b2]", reg) < 1)                 
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	return avSG3GPP_status;    
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function causes the instrument to perform a self-test and  */
/*			 returns the results.											 */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_SelfTest (ViSession instrSession, ViInt16 *testResult)
{
	ViStatus avSG3GPP_status = VI_SUCCESS;
	ViChar rdBuffer[BUFFER_SIZE];
    ViUInt32 retCount = 0;
    
    if ((avSG3GPP_status = viWrite (instrSession, "*TST?", 5, &retCount)) < 0)                
	    return avSG3GPP_status;                                                            
	if ((avSG3GPP_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCount)) < 0)        
	    return avSG3GPP_status;                                                            
	if (Scan (rdBuffer, "%s>%d[b2]", testResult) < 1)                 
	    return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	    
	return avSG3GPP_status; 
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_ErrorMessage (ViSession instrSession,
                                         ViStatus statusCode, ViChar errMessage[])
{
    ViStatus avSG3GPP_status = VI_SUCCESS;
    ViInt16 i;
    static avSG3GPP_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,     "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,        "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,    "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,  "WARNING: Error Query not supported"},     
        {VI_WARN_NSUP_REV_QUERY,    "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,    "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,   "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_INSTR_PARAMETER9 ,  "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10,  "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11,  "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12,  "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13,  "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14,  "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15,  "ERROR: Parameter 15 out of range"},
        {VI_ERROR_INSTR_PARAMETER16,  "ERROR: Parameter 16 out of range"},
        
        {VI_NULL, VI_NULL}
    };

    avSG3GPP_status = viStatusDesc (instrSession, statusCode, errMessage);
    if (avSG3GPP_status == VI_WARN_UNKNOWN_STATUS) {
        for (i = 0; statusDescArray[i].stringName; i++) {
            if (statusDescArray[i].stringVal == statusCode) {
                Fmt (errMessage, "%s<%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
            }
        }
        Fmt (errMessage, "%s<Unknown Error 0x%x[uw8p0]", statusCode);
        return (VI_WARN_UNKNOWN_STATUS);
    }
    
    avSG3GPP_status = VI_SUCCESS;
    return avSG3GPP_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Reset (ViSession instrSession)
{
    ViStatus avSG3GPP_status = VI_SUCCESS;
	ViUInt32 retCount = 0;  

    /*  Initialize the instrument to a known state.  */
    if ((avSG3GPP_status = viWrite (instrSession, "IP", 2, &retCount)) < 0)
        return avSG3GPP_status;

    return avSG3GPP_status;
    
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*===========================================================================*/
ViStatus _VI_FUNC avSG3GPP_Query_TypeRevision (ViSession instrSession,
                                               ViChar driverRev[], ViChar instrType[],
                                               ViChar instrCode[], ViChar instrRev[])
{
    ViStatus avSG3GPP_status = VI_SUCCESS;
	ViChar rdBuffer[BUFFER_SIZE]; 
	ViUInt32 retCount = 0;     
	
    if ((avSG3GPP_status = viWrite (instrSession, "IDN?", 4, &retCount)) < 0)
        return avSG3GPP_status;

    if ((avSG3GPP_status = viRead (instrSession, rdBuffer, BUFFER_SIZE, &retCount)) < 0)
        return avSG3GPP_status;

    if (Scan (rdBuffer, "%s>%s[t44],%s[dt44],%s[t44],%s", instrType, instrCode, instrRev) < 4)
        return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    Fmt (driverRev, "%s<%s", AVSG3GPP_REVISION);
    
    return avSG3GPP_status;    
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus CVIFUNC avSG3GPP_close (ViSession instrSession)
{
    ViSession rmSession;
    ViStatus avSG3GPP_status = VI_SUCCESS;

    if ((avSG3GPP_status = viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession)) < 0)
        return avSG3GPP_status;
            
    avSG3GPP_status = viClose (instrSession);
    viClose (rmSession);

    return avSG3GPP_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean avSG3GPP_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avSG3GPP_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avSG3GPP_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Unsigned Char Value Out Of Range - ViUInt8                      */
/* Purpose:  This function checks an unsigned char value to see if it        */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avSG3GPP_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16            */
/* Purpose:  This function checks a short unsigned integer value to see if it*/  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avSG3GPP_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avSG3GPP_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32                      */
/* Purpose:  This function checks a real (float) value to see if it lies     */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avSG3GPP_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avSG3GPP_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}


/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the avSG3GPP_Initialize function. */
/*			 When an error is detected this function is called to close the  */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from avSG3GPP_Initialize  */
/*			 to VI_NULL.                                                     */
/*===========================================================================*/
ViStatus avSG3GPP_initCleanUp (ViSession openRMSession,
                ViPSession openInstrSession, ViStatus currentStatus)
{
    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
